%  Copyright 2010 Carlos Gonzalez-Aguado
%
%  This file is part of the SAMPLE CODE for the following paper:
%  Bruche, M., Gonzalez-Aguado, C. "Recovery Rates, Default Probabilities, and the Credit Cycle"
%  Journal of Banking and Finance 34(4), 2010, 713-723.
%  
%  SAMPLE CODE is free software: you can redistribute it and/or modify it under
%  the terms of the GNU General Public License as published by the Free
%  Software Foundation, either version 3 of the License, or (at your option)
%  any later version.
%  
%  SAMPLE CODE is distributed in the hope that it will be useful, but WITHOUT
%  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%  more details.
%  
%  You should have received a copy of the GNU General Public License along with
%  SAMPLE CODE.  If not, see <http://www.gnu.org/licenses/>.


% ------ MAIN PROGRAM -------
% This is the base program for the likelihood maximization in Bruche and Gonzalez-Aguado (2010).

% Clear memory of previous variables
clear all
clear global
clc

% Double precision format
format long g

% Declare global variables
global pars dr_cycle rr_cycle dr_macro rr_macro rr_lag rr_ind macro_var macro_var_rr
global fil upd basic rr_mat0 rr_mat1 rr_pars basic

% Load data
gendata
data_prep
macro_var = macro;                 
macro_var_rr = macro_rr;
% In this example, we choose GDP to be the macro variable. Any of the other macro variables defined in 'data_prep' can be used. 


% --- MAXIMIZATION ---
% The key routine for maximizing the likelihood is 'maxiprogr' which takes five binary arguments corresponding to the inclusion in the model of (1) a latent variable in default rates, (2) macro variables in default rates, (3) latent variable in recovery rates, (4) macro variables in recovery rates, adn (5) industry variables in recovery rates.
% 'maxiprogr' returns 5 outputs: x that maximize the likelihood, the likelihood, the hessian, and AI and BI criterions. 

% -> Static model
[xsta,fsta,hsta,aicsta,bicsta] = maxiprogr(0,0,0,0,0);

% -> Simple model with latent cycle
[xcyc,fcyc,hcyc,aiccyc,biccyc] = maxiprogr(1,0,1,0,0);
aaa

% Simple model with macro
[xmac,fmac,hmac,aicmac,bicmac] = maxiprogr(0,1,0,1,0);

% Combined model (macro and cycle)
[xmacyc,fmacyc,hmacyc,aicmacyc,bicmacyc] = maxiprogr(1,1,1,1,0);

% Model with industry (no macro)
[xind,find,hind,aicind,bicind] = maxiprogr(1,0,1,0,1);

% Model with cycle only on default rates
[xdef,fdef,hdef,aicdef,bicdef] = maxiprogr(1,0,0,0,0);

% Model with cycle only in recoveries
[xrec,frec,hrec,aicrec,bicrec] = maxiprogr(0,0,1,0,0);


% -- Plots --
% - CORRELOGRAMS -
% STATIC
x = xsta;
hes = hsta;
fl = [fsta,aicsta,bicsta];
txtfil;
test
zsta = z;
zsta_up = z_up;
zsta_dw = z_dw;
correlogram;
cors_sta = cors;
% DYNAMIC
x = xcyc;
hes = hcyc;
fl = [fcyc,aiccyc,biccyc];
txtfil;
test;
zcyc = z;
zcyc_up = z_up;
zcyc_dw = z_dw;
correlogram;
cors_cyc = cors;
% Plot correlogram 
figure;
subplot(2,1,1)
bar(cors_sta,'w')
xlim([0 lags + 1])
ylim([-0.1 0.25])
hold on
plot(linspace(0,41,1000),1.96/sqrt(n),'r');
plot(linspace(0,41,1000),-1.96/sqrt(n),'r');
subplot(2,1,2)
bar(cors_cyc,'w')
xlim([0 lags + 1])
ylim([-0.1 0.25])
hold on
plot(linspace(0,41,1000),1.96/sqrt(n),'r');
plot(linspace(0,41,1000),-1.96/sqrt(n),'r');
% - QQPLOTS -
qqplot


